@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -144,8 +144,6 @@ D2ParserContext::contextName()
         return("dns-servers");
     case CONTROL_SOCKET:
         return("control-socket");
-    case LOGGING:
-        return ("Logging");
     case LOGGERS:
         return ("loggers");
     case OUTPUT_OPTIONS:
