@@ -25,11 +25,11 @@
 #include <lease_cmds.h>
 #include <lease_parser.h>
 #include <lease_cmds_log.h>
+#include <stats/stats_mgr.h>
 #include <util/encode/hex.h>
 #include <util/multi_threading_mgr.h>
 #include <util/strutil.h>
 
-#include <boost/bind.hpp>
 #include <boost/scoped_ptr.hpp>
 #include <boost/algorithm/string.hpp>
 #include <string>
@@ -41,6 +41,7 @@ using namespace isc::dhcp_ddns;
 using namespace isc::config;
 using namespace isc::asiolink;
 using namespace isc::hooks;
+using namespace isc::stats;
 using namespace isc::util;
 using namespace std;
 
@@ -83,7 +84,9 @@ public:
         /// Supported values are: "address", hw-address and duid.
         ///
         /// @param txt text to be converted
+        ///
         /// @return value converted to Parameters::Type
+        ///
         /// @throw BadValue if unsupported type is specified
         static Type txtToType(const std::string& txt) {
             if (txt == "address") {
@@ -129,6 +132,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// add command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     leaseAddHandler(CalloutHandle& handle);
@@ -140,6 +144,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// add command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     lease6BulkApplyHandler(CalloutHandle& handle);
@@ -150,6 +155,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// get command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     leaseGetHandler(CalloutHandle& handle);
@@ -163,6 +169,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// get command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise.
     int
     leaseGetAllHandler(CalloutHandle& handle);
@@ -179,6 +186,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// get commands JSON text in the "command" argument.
+    ///
     /// @return 0 if the handler has been invoked successfully, 1 if an
     /// error occurs, 3 if no leases are returned.
     int
@@ -190,6 +198,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// get command JSON text in the "command" argument
+    ///
     /// @return 0 if the handler has been invoked successfully, 1 if an
     /// error occurs, 3 if no leases are returned.
     int
@@ -201,6 +210,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// get command JSON text in the "command" argument
+    ///
     /// @return 0 if the handler has been invoked successfully, 1 if an
     /// error occurs, 3 if no leases are returned.
     int
@@ -212,6 +222,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// get command JSON text in the "command" argument
+    ///
     /// @return 0 if the handler has been invoked successfully, 1 if an
     /// error occurs, 3 if no leases are returned.
     int
@@ -224,6 +235,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// get command JSON text in the "command" argument
+    ///
     /// @return 0 if the handler has been invoked successfully, 1 if an
     /// error occurs, 3 if no leases are returned.
     int
@@ -235,6 +247,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// delete command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     lease4DelHandler(CalloutHandle& handle);
@@ -245,6 +258,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// delete command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     lease6DelHandler(CalloutHandle& handle);
@@ -255,6 +269,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// update command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     lease4UpdateHandler(CalloutHandle& handle);
@@ -265,6 +280,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// update command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     lease6UpdateHandler(CalloutHandle& handle);
@@ -275,6 +291,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// wipe command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     lease4WipeHandler(CalloutHandle& handle);
@@ -285,6 +302,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// wipe command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int
     lease6WipeHandler(CalloutHandle& handle);
@@ -295,6 +313,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// lease4-resend-ddns command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int lease4ResendDdnsHandler(CalloutHandle& handle);
 
@@ -304,6 +323,7 @@ public:
     ///
     /// @param handle Callout context - which is expected to contain the
     /// lease6-resend-ddns command JSON text in the "command" argument
+    ///
     /// @return 0 upon success, non-zero otherwise
     int lease6ResendDdnsHandler(CalloutHandle& handle);
 
@@ -314,7 +334,9 @@ public:
     ///
     /// @param v6 whether addresses allowed are v4 (false) or v6 (true)
     /// @param args arguments passed to command
+    ///
     /// @return parsed parameters
+    ///
     /// @throw BadValue if input arguments don't make sense.
     Parameters getParameters(bool v6, const ConstElementPtr& args);
 
@@ -349,6 +371,8 @@ public:
     /// @param control_result Control result: "empty" of the lease was
     /// not found, "error" otherwise.
     /// @param error_message Error message.
+    ///
+    /// @return The JSON map of the failed leases.
     ElementPtr createFailedLeaseMap(const Lease::Type& lease_type,
                                     const IOAddress& lease_address,
                                     const DuidPtr& duid,
@@ -356,16 +380,331 @@ public:
                                     const std::string& error_message) const;
 
     /// @brief Fetches an IP address parameter from a map of parameters
+    ///
     /// @param map of parameters in which to look
     /// @name name of the parameter desired
     /// @param family expected protocol family of the address parameter,
     /// AF_INET or AF_INET6
+    ///
     /// @return IOAddress containing the value of the parameter.
+    ///
     /// @throw BadValue if the parameter is missing or invalid
     IOAddress getAddressParam(ConstElementPtr params, const std::string name,
                               short family = AF_INET) const;
+
+    /// @brief Update stats when adding lease.
+    ///
+    /// @param lease Added lease.
+    static void updateStatsOnAdd(const Lease4Ptr& lease);
+
+    /// @brief Update stats when adding lease.
+    ///
+    /// @param lease Added lease.
+    static void updateStatsOnAdd(const Lease6Ptr& lease);
+
+    /// @brief Update stats when updating lease.
+    ///
+    /// @param existing Lease data before update.
+    /// @param lease Lease data after update.
+    static void updateStatsOnUpdate(const Lease4Ptr& existing,
+                                    const Lease4Ptr& lease);
+
+    /// @brief Update stats when updating lease.
+    ///
+    /// @param existing Lease data before update.
+    /// @param lease Lease data after update.
+    static void updateStatsOnUpdate(const Lease6Ptr& existing,
+                                    const Lease6Ptr& lease);
+
+    /// @brief Update stats when deleting lease.
+    ///
+    /// @param lease Deleted lease.
+    static void updateStatsOnDelete(const Lease4Ptr& lease);
+
+    /// @brief Update stats when deleting lease.
+    ///
+    /// @param lease Deleted lease.
+    static void updateStatsOnDelete(const Lease6Ptr& lease);
+
+    /// @brief Add or update lease.
+    ///
+    /// @param lease The lease to be added or updated (if exists).
+    /// @param force_create Flag to indicate if the function should try to
+    /// create the lease if it does not exists, or simply update and fail in
+    /// such case.
+    ///
+    /// @return true if lease has been successfully added, false otherwise.
+    static bool addOrUpdate4(Lease4Ptr lease, bool force_create);
+
+    /// @brief Add or update lease.
+    ///
+    /// @param lease The lease to be added or updated (if exists).
+    /// @param force_create Flag to indicate if the function should try to
+    /// create the lease if it does not exists, or simply update and fail in
+    /// such case.
+    ///
+    /// @return true if lease has been successfully added, false otherwise.
+    static bool addOrUpdate6(Lease6Ptr lease, bool force_create);
 };
 
+void
+LeaseCmdsImpl::updateStatsOnAdd(const Lease4Ptr& lease) {
+    if (!lease->stateExpiredReclaimed()) {
+        StatsMgr::instance().addValue(
+            StatsMgr::generateName("subnet", lease->subnet_id_,
+                                   "assigned-addresses"),
+            int64_t(1));
+        if (lease->stateDeclined()) {
+            StatsMgr::instance().addValue("declined-addresses", int64_t(1));
+
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", lease->subnet_id_,
+                                       "declined-addresses"),
+                int64_t(1));
+        }
+    }
+}
+
+void
+LeaseCmdsImpl::updateStatsOnAdd(const Lease6Ptr& lease) {
+    if (!lease->stateExpiredReclaimed()) {
+        StatsMgr::instance().addValue(
+            StatsMgr::generateName("subnet", lease->subnet_id_,
+                                   lease->type_ == Lease::TYPE_NA ?
+                                   "assigned-nas" : "assigned-pds"),
+            int64_t(1));
+        if (lease->stateDeclined()) {
+            StatsMgr::instance().addValue("declined-addresses", int64_t(1));
+
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", lease->subnet_id_,
+                                       "declined-addresses"),
+                int64_t(1));
+        }
+    }
+}
+
+void
+LeaseCmdsImpl::updateStatsOnUpdate(const Lease4Ptr& existing,
+                                   const Lease4Ptr& lease) {
+    if (!existing->stateExpiredReclaimed()) {
+        // old lease is non expired-reclaimed
+        if (existing->subnet_id_ != lease->subnet_id_) {
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", existing->subnet_id_,
+                                       "assigned-addresses"),
+                int64_t(-1));
+        }
+        if (existing->stateDeclined()) {
+            // old lease is declined
+            StatsMgr::instance().addValue("declined-addresses", int64_t(-1));
+
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", existing->subnet_id_,
+                                       "declined-addresses"),
+                int64_t(-1));
+        }
+        if (!lease->stateExpiredReclaimed()) {
+            // new lease is non expired-reclaimed
+            if (existing->subnet_id_ != lease->subnet_id_) {
+                StatsMgr::instance().addValue(
+                    StatsMgr::generateName("subnet", lease->subnet_id_,
+                                           "assigned-addresses"),
+                    int64_t(1));
+            }
+            if (lease->stateDeclined()) {
+                // new lease is declined
+                StatsMgr::instance().addValue("declined-addresses", int64_t(1));
+
+                StatsMgr::instance().addValue(
+                    StatsMgr::generateName("subnet", lease->subnet_id_,
+                                           "declined-addresses"),
+                    int64_t(1));
+            }
+        }
+    } else {
+        // old lease is expired-reclaimed
+        if (!lease->stateExpiredReclaimed()) {
+            // new lease is non expired-reclaimed
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", lease->subnet_id_,
+                                       "assigned-addresses"),
+                int64_t(1));
+            if (lease->stateDeclined()) {
+                // new lease is declined
+                StatsMgr::instance().addValue("declined-addresses", int64_t(1));
+
+                StatsMgr::instance().addValue(
+                    StatsMgr::generateName("subnet", lease->subnet_id_,
+                                           "declined-addresses"),
+                    int64_t(1));
+            }
+        }
+    }
+}
+
+void
+LeaseCmdsImpl::updateStatsOnUpdate(const Lease6Ptr& existing,
+                                   const Lease6Ptr& lease) {
+    if (!existing->stateExpiredReclaimed()) {
+        // old lease is non expired-reclaimed
+        if (existing->subnet_id_ != lease->subnet_id_) {
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", existing->subnet_id_,
+                                       lease->type_ == Lease::TYPE_NA ?
+                                       "assigned-nas" : "assigned-pds"),
+                int64_t(-1));
+        }
+        if (existing->stateDeclined()) {
+            // old lease is declined
+            StatsMgr::instance().addValue("declined-addresses", int64_t(-1));
+
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", existing->subnet_id_,
+                                       "declined-addresses"),
+                int64_t(-1));
+        }
+        if (!lease->stateExpiredReclaimed()) {
+            // new lease is non expired-reclaimed
+            if (existing->subnet_id_ != lease->subnet_id_) {
+                StatsMgr::instance().addValue(
+                    StatsMgr::generateName("subnet", lease->subnet_id_,
+                                           lease->type_ == Lease::TYPE_NA ?
+                                           "assigned-nas" : "assigned-pds"),
+                    int64_t(1));
+            }
+            if (lease->stateDeclined()) {
+                // new lease is declined
+                StatsMgr::instance().addValue("declined-addresses", int64_t(1));
+
+                StatsMgr::instance().addValue(
+                    StatsMgr::generateName("subnet", lease->subnet_id_,
+                                           "declined-addresses"),
+                    int64_t(1));
+            }
+        }
+    } else {
+        // old lease is expired-reclaimed
+        if (!lease->stateExpiredReclaimed()) {
+            // new lease is non expired-reclaimed
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", lease->subnet_id_,
+                                       lease->type_ == Lease::TYPE_NA ?
+                                       "assigned-nas" : "assigned-pds"),
+                int64_t(1));
+            if (lease->stateDeclined()) {
+                // new lease is declined
+                StatsMgr::instance().addValue("declined-addresses", int64_t(1));
+
+                StatsMgr::instance().addValue(
+                    StatsMgr::generateName("subnet", lease->subnet_id_,
+                                           "declined-addresses"),
+                    int64_t(1));
+            }
+        }
+    }
+}
+
+void
+LeaseCmdsImpl::updateStatsOnDelete(const Lease4Ptr& lease) {
+    if (!lease->stateExpiredReclaimed()) {
+        StatsMgr::instance().addValue(
+            StatsMgr::generateName("subnet", lease->subnet_id_,
+                                   "assigned-addresses"),
+            int64_t(-1));
+        if (lease->stateDeclined()) {
+            StatsMgr::instance().addValue("declined-addresses", int64_t(-1));
+
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", lease->subnet_id_,
+                                       "declined-addresses"),
+                int64_t(-1));
+        }
+    }
+}
+
+void
+LeaseCmdsImpl::updateStatsOnDelete(const Lease6Ptr& lease) {
+    if (!lease->stateExpiredReclaimed()) {
+        StatsMgr::instance().addValue(
+            StatsMgr::generateName("subnet", lease->subnet_id_,
+                                   lease->type_ == Lease::TYPE_NA ?
+                                   "assigned-nas" : "assigned-pds"),
+            int64_t(-1));
+        if (lease->stateDeclined()) {
+            StatsMgr::instance().addValue("declined-addresses", int64_t(-1));
+
+            StatsMgr::instance().addValue(
+                StatsMgr::generateName("subnet", lease->subnet_id_,
+                                       "declined-addresses"),
+                int64_t(-1));
+        }
+    }
+}
+
+bool
+LeaseCmdsImpl::addOrUpdate4(Lease4Ptr lease, bool force_create) {
+    Lease4Ptr existing = LeaseMgrFactory::instance().getLease4(lease->addr_);
+    if (force_create && !existing) {
+        // lease does not exist
+        if (!LeaseMgrFactory::instance().addLease(lease)) {
+            isc_throw(db::DuplicateEntry,
+                      "lost race between calls to get and add");
+        }
+        LeaseCmdsImpl::updateStatsOnAdd(lease);
+        return (true);
+    }
+    if (existing) {
+        // Update lease current expiration time with value received from the
+        // database. Some database backends reject operations on the lease if
+        // the current expiration time value does not match what is stored.
+        Lease::syncCurrentExpirationTime(*existing, *lease);
+    }
+    try {
+        LeaseMgrFactory::instance().updateLease4(lease);
+    } catch (const NoSuchLease&) {
+        isc_throw(InvalidOperation, "failed to update the lease with address "
+                  << lease->addr_ << " either because the lease has been "
+                  "deleted or it has changed in the database, in both cases a "
+                  "retry might succeed");
+    }
+
+    LeaseCmdsImpl::updateStatsOnUpdate(existing, lease);
+    return (false);
+}
+
+bool
+LeaseCmdsImpl::addOrUpdate6(Lease6Ptr lease, bool force_create) {
+    Lease6Ptr existing =
+        LeaseMgrFactory::instance().getLease6(lease->type_, lease->addr_);
+    if (force_create && !existing) {
+        // lease does not exist
+        if (!LeaseMgrFactory::instance().addLease(lease)) {
+            isc_throw(db::DuplicateEntry,
+                      "lost race between calls to get and add");
+        }
+        LeaseCmdsImpl::updateStatsOnAdd(lease);
+        return (true);
+    }
+    if (existing) {
+        // Update lease current expiration time with value received from the
+        // database. Some database backends reject operations on the lease if
+        // the current expiration time value does not match what is stored.
+        Lease::syncCurrentExpirationTime(*existing, *lease);
+    }
+    try {
+        LeaseMgrFactory::instance().updateLease6(lease);
+    } catch (const NoSuchLease&) {
+        isc_throw(InvalidOperation, "failed to update the lease with address "
+                  << lease->addr_ << " either because the lease has been "
+                  "deleted or it has changed in the database, in both cases a "
+                  "retry might succeed");
+    }
+
+    LeaseCmdsImpl::updateStatsOnUpdate(existing, lease);
+    return (false);
+}
+
 int
 LeaseCmdsImpl::leaseAddHandler(CalloutHandle& handle) {
     // Arbitrary defaulting to DHCPv4 or with other words extractCommand
@@ -420,6 +759,7 @@ LeaseCmdsImpl::leaseAddHandler(CalloutHandle& handle) {
                 if (!success) {
                     isc_throw(db::DuplicateEntry, "IPv4 lease already exists.");
                 }
+                LeaseCmdsImpl::updateStatsOnAdd(lease4);
                 resp << "Lease for address " << lease4->addr_.toText()
                      << ", subnet-id " << lease4->subnet_id_ << " added.";
             }
@@ -454,6 +794,7 @@ LeaseCmdsImpl::leaseAddHandler(CalloutHandle& handle) {
                 if (!success) {
                     isc_throw(db::DuplicateEntry, "IPv6 lease already exists.");
                 }
+                LeaseCmdsImpl::updateStatsOnAdd(lease6);
                 if (lease6->type_ == Lease::TYPE_NA) {
                     resp << "Lease for address " << lease6->addr_.toText()
                          << ", subnet-id " << lease6->subnet_id_ << " added.";
@@ -881,7 +1222,7 @@ LeaseCmdsImpl::leaseGetPageHandler(CalloutHandle& handle) {
                          s.str(), args);
         setResponse(handle, response);
 
-    } catch (std::exception& ex) {
+    } catch (const std::exception& ex) {
         setErrorResponse(handle, ex.what());
         return (CONTROL_RESULT_ERROR);
     }
@@ -1167,6 +1508,7 @@ LeaseCmdsImpl::lease4DelHandler(CalloutHandle& handle) {
 
         if (LeaseMgrFactory::instance().deleteLease(lease4)) {
             setSuccessResponse(handle, "IPv4 lease deleted.");
+            LeaseCmdsImpl::updateStatsOnDelete(lease4);
         } else {
             setErrorResponse (handle, "IPv4 lease not found.", CONTROL_RESULT_EMPTY);
         }
@@ -1184,24 +1526,6 @@ LeaseCmdsImpl::lease4DelHandler(CalloutHandle& handle) {
     return (0);
 }
 
-namespace { // anonymous namespace.
-
-void updateOrAdd(Lease6Ptr lease) {
-    try {
-        // Try to update.
-        LeaseMgrFactory::instance().updateLease6(lease);
-
-    } catch (const NoSuchLease& ex) {
-        // Lease to be updated not found, so add it.
-        if (!LeaseMgrFactory::instance().addLease(lease)) {
-            isc_throw(db::DuplicateEntry,
-                      "lost race between calls to update and add");
-        }
-    }
-}
-
-} // end of anonymous namespace.
-
 int
 LeaseCmdsImpl::lease6BulkApplyHandler(CalloutHandle& handle) {
     try {
@@ -1289,6 +1613,7 @@ LeaseCmdsImpl::lease6BulkApplyHandler(CalloutHandle& handle) {
                         // leases.
                         if (LeaseMgrFactory::instance().deleteLease(lease)) {
                             ++success_count;
+                            LeaseCmdsImpl::updateStatsOnDelete(lease);
 
                         } else {
                             // Lazy creation of the list of leases which failed to delete.
@@ -1328,8 +1653,6 @@ LeaseCmdsImpl::lease6BulkApplyHandler(CalloutHandle& handle) {
             // Iterate over all leases.
             for (auto lease : parsed_leases_list) {
 
-                Lease6Parser parser;
-
                 try {
                     if (MultiThreadingMgr::instance().getMode() &&
                         !MultiThreadingMgr::instance().isInCriticalSection()) {
@@ -1340,17 +1663,17 @@ LeaseCmdsImpl::lease6BulkApplyHandler(CalloutHandle& handle) {
                             use_cs = !resource_handler.tryLock(lease->type_,
                                                                lease->addr_);
                             if (!use_cs) {
-                                updateOrAdd(lease);
+                                addOrUpdate6(lease, true);
                             }
                         }
                         if (use_cs) {
                             // Failed to avoid the race.
                             MultiThreadingCriticalSection cs;
-                            updateOrAdd(lease);
+                            addOrUpdate6(lease, true);
                         }
                     } else {
                         // No multi-threading.
-                        updateOrAdd(lease);
+                        addOrUpdate6(lease, true);
                     }
 
                     ++success_count;
@@ -1451,6 +1774,7 @@ LeaseCmdsImpl::lease6DelHandler(CalloutHandle& handle) {
 
         if (LeaseMgrFactory::instance().deleteLease(lease6)) {
             setSuccessResponse(handle, "IPv6 lease deleted.");
+            LeaseCmdsImpl::updateStatsOnDelete(lease6);
         } else {
             setErrorResponse (handle, "IPv6 lease not found.", CONTROL_RESULT_EMPTY);
         }
@@ -1468,22 +1792,6 @@ LeaseCmdsImpl::lease6DelHandler(CalloutHandle& handle) {
     return (0);
 }
 
-namespace { // anonymous namepace.
-
-bool addOrUpdate4(Lease4Ptr lease, bool force_create) {
-    if (force_create && !LeaseMgrFactory::instance().getLease4(lease->addr_)) {
-        if (!LeaseMgrFactory::instance().addLease(lease)) {
-            isc_throw(db::DuplicateEntry,
-                      "lost race between calls to get and add");
-        }
-        return (true);
-    }
-    LeaseMgrFactory::instance().updateLease4(lease);
-    return (false);
-}
-
-} // end of anonymous namespace.
-
 int
 LeaseCmdsImpl::lease4UpdateHandler(CalloutHandle& handle) {
     try {
@@ -1537,23 +1845,6 @@ LeaseCmdsImpl::lease4UpdateHandler(CalloutHandle& handle) {
     return (0);
 }
 
-namespace { // anonymous namepace.
-
-bool addOrUpdate6(Lease6Ptr lease, bool force_create) {
-    if (force_create &&
-        !LeaseMgrFactory::instance().getLease6(lease->type_, lease->addr_)) {
-        if (!LeaseMgrFactory::instance().addLease(lease)) {
-            isc_throw(db::DuplicateEntry,
-                      "lost race between calls to get and add");
-        }
-        return (true);
-    }
-    LeaseMgrFactory::instance().updateLease6(lease);
-    return (false);
-}
-
-} // end of anonymous namespace.
-
 int
 LeaseCmdsImpl::lease6UpdateHandler(CalloutHandle& handle) {
     try {
@@ -1625,9 +1916,28 @@ LeaseCmdsImpl::lease4WipeHandler(CalloutHandle& handle) {
         }
 
         if (id) {
-            // Wipe a single subnet
+            // Wipe a single subnet.
             num = LeaseMgrFactory::instance().wipeLeases4(id);
             ids << " " << id;
+
+            auto observation = StatsMgr::instance().getObservation(
+                StatsMgr::generateName("subnet", id, "declined-addresses"));
+
+            int64_t previous_declined = 0;
+
+            if (observation) {
+                previous_declined = observation->getInteger().first;
+            }
+
+            StatsMgr::instance().setValue(
+                StatsMgr::generateName("subnet", id, "assigned-addresses"),
+                int64_t(0));
+
+            StatsMgr::instance().setValue(
+                StatsMgr::generateName("subnet", id, "declined-addresses"),
+                int64_t(0));
+
+            StatsMgr::instance().addValue("declined-addresses", -previous_declined);
         } else {
             // Wipe them all!
             ConstSrvConfigPtr config = CfgMgr::instance().getCurrentCfg();
@@ -1638,7 +1948,16 @@ LeaseCmdsImpl::lease4WipeHandler(CalloutHandle& handle) {
             for (auto sub : *subs) {
                 num += LeaseMgrFactory::instance().wipeLeases4(sub->getID());
                 ids << " " << sub->getID();
+                StatsMgr::instance().setValue(
+                    StatsMgr::generateName("subnet", sub->getID(), "assigned-addresses"),
+                    int64_t(0));
+
+                StatsMgr::instance().setValue(
+                    StatsMgr::generateName("subnet", sub->getID(), "declined-addresses"),
+                    int64_t(0));
             }
+
+            StatsMgr::instance().setValue("declined-addresses", int64_t(0));
         }
 
         stringstream tmp;
@@ -1680,6 +1999,29 @@ LeaseCmdsImpl::lease6WipeHandler(CalloutHandle& handle) {
             // Wipe a single subnet.
             num = LeaseMgrFactory::instance().wipeLeases6(id);
             ids << " " << id;
+
+            auto observation = StatsMgr::instance().getObservation(
+                StatsMgr::generateName("subnet", id, "declined-addresses"));
+
+            int64_t previous_declined = 0;
+
+            if (observation) {
+                previous_declined = observation->getInteger().first;
+            }
+
+            StatsMgr::instance().setValue(
+                StatsMgr::generateName("subnet", id, "assigned-nas" ),
+                int64_t(0));
+
+            StatsMgr::instance().setValue(
+                StatsMgr::generateName("subnet", id, "assigned-pds"),
+                int64_t(0));
+
+            StatsMgr::instance().setValue(
+                StatsMgr::generateName("subnet", id, "declined-addresses"),
+                int64_t(0));
+
+            StatsMgr::instance().addValue("declined-addresses", -previous_declined);
        } else {
             // Wipe them all!
             ConstSrvConfigPtr config = CfgMgr::instance().getCurrentCfg();
@@ -1690,7 +2032,20 @@ LeaseCmdsImpl::lease6WipeHandler(CalloutHandle& handle) {
             for (auto sub : *subs) {
                 num += LeaseMgrFactory::instance().wipeLeases6(sub->getID());
                 ids << " " << sub->getID();
+                StatsMgr::instance().setValue(
+                    StatsMgr::generateName("subnet", sub->getID(), "assigned-nas" ),
+                    int64_t(0));
+
+                StatsMgr::instance().setValue(
+                    StatsMgr::generateName("subnet", sub->getID(), "assigned-pds"),
+                    int64_t(0));
+
+                StatsMgr::instance().setValue(
+                    StatsMgr::generateName("subnet", sub->getID(), "declined-addresses"),
+                    int64_t(0));
             }
+
+            StatsMgr::instance().setValue("declined-addresses", int64_t(0));
         }
 
         stringstream tmp;
