@@ -1,4 +1,4 @@
-// Copyright (C) 2017-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2017-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -8,7 +8,7 @@
 
 #include <cc/data.h>
 #include <cc/json_feed.h>
-#include <boost/bind.hpp>
+#include <functional>
 
 using namespace isc::data;
 using namespace isc::util;
@@ -33,7 +33,7 @@ const int JSONFeed::FEED_OK_EVT;
 const int JSONFeed::FEED_FAILED_EVT;
 
 JSONFeed::JSONFeed()
-    : StateModel(), buffer_(), error_message_(), open_scopes_(0),
+    : StateModel(), buffer_(), data_ptr_(0), error_message_(), open_scopes_(0),
       output_() {
 }
 
@@ -100,8 +100,9 @@ JSONFeed::postBuffer(const void* buf, const size_t buf_size) {
         if (getNextEvent() == NEED_MORE_DATA_EVT) {
             transition(getCurrState(), MORE_DATA_PROVIDED_EVT);
         }
-        buffer_.insert(buffer_.end(), static_cast<const char*>(buf),
+        buffer_.assign(static_cast<const char*>(buf),
                        static_cast<const char*>(buf) + buf_size);
+        data_ptr_ = 0;
     }
 }
 
@@ -134,17 +135,17 @@ JSONFeed::defineStates() {
     StateModel::defineStates();
 
     defineState(RECEIVE_START_ST, "RECEIVE_START_ST",
-                boost::bind(&JSONFeed::receiveStartHandler, this));
+                std::bind(&JSONFeed::receiveStartHandler, this));
     defineState(WHITESPACE_BEFORE_JSON_ST, "WHITESPACE_BEFORE_JSON_ST",
-                boost::bind(&JSONFeed::whiteSpaceBeforeJSONHandler, this));
+                std::bind(&JSONFeed::whiteSpaceBeforeJSONHandler, this));
     defineState(INNER_JSON_ST, "INNER_JSON_ST",
-                boost::bind(&JSONFeed::innerJSONHandler, this));
+                std::bind(&JSONFeed::innerJSONHandler, this));
     defineState(STRING_JSON_ST, "STRING_JSON_ST",
-                boost::bind(&JSONFeed::stringJSONHandler, this));
+                std::bind(&JSONFeed::stringJSONHandler, this));
     defineState(ESCAPE_JSON_ST, "ESCAPE_JSON_ST",
-                boost::bind(&JSONFeed::escapeJSONHandler, this));
+                std::bind(&JSONFeed::escapeJSONHandler, this));
     defineState(JSON_END_ST, "JSON_END_ST",
-                boost::bind(&JSONFeed::endJSONHandler, this));
+                std::bind(&JSONFeed::endJSONHandler, this));
 }
 
 void
@@ -163,9 +164,8 @@ JSONFeed::onModelFailure(const std::string& explanation) {
 bool
 JSONFeed::popNextFromBuffer(char& next) {
     // If there are any characters in the buffer, pop next.
-    if (!buffer_.empty()) {
-        next = buffer_.front();
-        buffer_.pop_front();
+    if (!buffer_.empty() && (data_ptr_ < buffer_.size())) {
+        next = buffer_[data_ptr_++];
         return (true);
     }
     return (false);
@@ -241,6 +241,7 @@ JSONFeed::receiveStartHandler() {
                 feedFailure("invalid first character " + std::string(1, c));
                 break;
             }
+            break;
 
         default:
             invalidEventError("receiveStartHandler", getNextEvent());
@@ -294,7 +295,7 @@ JSONFeed::innerJSONHandler() {
                 transition(JSON_END_ST, FEED_OK_EVT);
 
             } else {
-                transition(getCurrState(), DATA_READ_OK_EVT);
+                postNextEvent(DATA_READ_OK_EVT);
             }
             break;
 
@@ -303,7 +304,7 @@ JSONFeed::innerJSONHandler() {
             break;
 
         default:
-            transition(getCurrState(), DATA_READ_OK_EVT);
+            postNextEvent(DATA_READ_OK_EVT);
         }
     }
 }
